/lang.defaultnames [
  [ "am" "am_ET" "Amharic" ]
  [ "ar" "ar_EG" "Arabic" ]
  [ "ast" "ast_ES" "Asturianu" ]
  [ "be" "be_BY" "Беларуская" ]
  [ "bg" "bg_BG" "Български" ]
  [ "bn" "bn_BD" "Bengali" ]
  [ "bo" "bo_IN" "Tibetan" ]
  [ "bs" "bs_BA" "Bosanski" ]
  [ "ca" "ca_ES" "Català" ]
  [ "cs" "cs_CZ" "Čeština" ]
  [ "da" "da_DK" "Dansk" ]
  [ "de" "de_DE" "Deutsch" ]
  [ "dz" "dz_BT" "Dzongkha" ]
  [ "el" "el_GR" "Ελληνικά" ]
  [ "en" "en_US" "English" ]
  [ "eo" "eo" "Esperanto" ]
  [ "es" "es_ES" "Español" ]
  [ "et" "et_EE" "Eesti" ]
  [ "eu" "eu_ES" "Euskara" ]
  [ "fa" "fa_IR" "فارسی" ]
  [ "fi" "fi_FI" "Suomi" ]
  [ "fr" "fr_FR" "Français" ]
  [ "ga" "ga_IE" "Gaeilge" ]
  [ "gl" "gl_ES" "Galego" ]
  [ "gu" "gu_IN" "Gujarati" ]
  [ "he" "he_IL" "תירבע" ]
  [ "hi" "hi_IN" "Hindi" ]
  [ "hr" "hr_HR" "Hrvatski" ]
  [ "hu" "hu_HU" "Magyar" ]
  [ "id" "id_ID" "Bahasa Indonesia" ]
  [ "is" "is_IS" "Íslenska" ]
  [ "it" "it_IT" "Italiano" ]
  [ "ja" "ja_JP" "日本語" ]
  [ "ka" "ka_GE" "ქართული" ]
  [ "kk" "kk_KZ" "Қазақ" ]
  [ "km" "km_KH" "Khmer" ]
  [ "kn" "kn_IN" "ಕನ್ನಡ" ]
  [ "ko" "ko_KR" "한국어" ]
  [ "ku" "ku_TR" "Kurdî" ]
  [ "lo" "lo_LA" "Lao" ]
  [ "lt" "lt_LT" "Lietuviškai" ]
  [ "lv" "lv_LV" "Latviski" ]
  [ "mk" "mk_MK" "Македонски" ]
  [ "ml" "ml_IN" "Malayalam" ]
  [ "mr" "mr_IN" "Marathi" ]
  [ "my" "my_MM" "Burmese" ]
  [ "ne" "ne_NP" "Nepali" ]
  [ "nl" "nl_NL" "Nederlands" ]
  [ "nb" "nb_NO" "Norsk bokmål" ]
  [ "nn" "nn_NO" "Norsk nynorsk" ]
  [ "pa" "pa_IN" "Punjabi (Gurmukhi)" ]
  [ "pl" "pl_PL" "Polski" ]
  [ "pt_BR" "pt_BR" "Português do Brasil" ]
  [ "pt_PT" "pt_PT" "Português" ]
  [ "ro" "ro_RO" "Română" ]
  [ "ru" "ru_RU" "Русский" ]
  [ "se" "se_NO" "Sámegillii" ]
  [ "si" "si_LK" "සිංහල" ]
  [ "sk" "sk_SK" "Slovenčina" ]
  [ "sl" "sl_SI" "Slovenščina" ]
  [ "sq" "sq_AL" "Shqip" ]
  [ "sr" "sr_RS" "Српски" ]
  [ "sv" "sv_SE" "Svenska" ]
  [ "ta" "ta_IN" "Tamil" ]
  [ "te" "te_IN" "తెలుగు" ]
  [ "th" "th_TH" "Thai" ]
  [ "tl" "tl_PH" "Tagalog" ]
  [ "tr" "tr_TR" "Türkçe" ]
  [ "ug" "ug_CN" "Uyghur" ]
  [ "uk" "uk_UA" "Українська" ]
  [ "vi" "vi_VN" "Tiếng Việt" ]
  [ "zh_CN" "zh_CN" "中文(简体)" ]
  [ "zh_TW" "zh_TW" "中文(繁體)" ]
] def
